/*
 * @(#) JHLauncher.java 1.37 04/11/99 SMI
 *
 * Copyright (c) 1997-2001 by Sun Microsystems, Inc.
 * All rights reserved.
 *
 */

import java.util.Locale;
import java.net.*;
import javax.help.*;
import java.util.Vector;
import java.util.Enumeration;
import java.io.*;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class displays JavaHelp HelpSets.
 *
  * @author Roger D. Brinkley
  */

public class JHLauncher {

    HelpSet hs = null;
    private JHelp jh = null;
    private JFrame frame= null;
    private static ClassLoader loader=null;
    private static String id=null;

    // The initial width and height of the frame
    public static int viewerWidth = 645;
    public static int viewerHeight = 495;

    private static String hsName = null; // name for the HelpSet 
    private static URL hsURL = null; // name of the HelpSet URL
    private static String hsPath = null; // URL spec to the HelpSet

    public JHLauncher() {
	if (debug) {
	    debug("hsName: "+hsName);
	    debug("hsURL: "+hsURL);
	    debug("loader: "+loader);
	}

	if (hsURL != null) {
	    initialize(hsURL, loader);
	} else {
	    initialize(hsName, loader);
	}
	if (hs == null) {
	    return;
	}

	WindowListener closer = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
	        System.exit(0);
	    }
	    public void windowClosed(WindowEvent e) {
	        System.exit(0);
	    }
	};

	if (id != null) {
	  try {
	    jh.setCurrentID(id);
	  } catch (BadIDException e) {
	    // ignore
	  }
	}

	frame = new JFrame((hsURL != null) ? hsURL.toString() : hsName);
	frame.addWindowListener(closer);
	frame.getContentPane().add(jh);	// the JH panel
	frame.pack();
	frame.setSize(viewerWidth, viewerHeight);
	frame.show();
    }

    protected void initialize(String name, ClassLoader loader) {
	URL url = HelpSet.findHelpSet(loader, name, "", Locale.getDefault());
	if (url == null) {
	    url = HelpSet.findHelpSet(loader,name, ".hs", Locale.getDefault());
	}
	initialize(url, loader);
    }

    protected void initialize(URL url, ClassLoader loader) {
	try {
	    hs = new HelpSet(loader, url);
	} catch (Exception ee) {
	    System.out.println("HelpSet Not Found");
	    return;
	}
	// The JavaHelp can't be added to a BorderLayout because it
	// isnt' a component. For this demo we'll use the embeded method
	// since we don't want a Frame to be created.

	jh = new JHelp(hs);
    }

    /**
     * Parse the arguments
     */
    private static String[] shiftArgs(String args[], int step) {
	int count = args.length;
	String back[] = new String[count-step];
	for (int i=0; i<count-step; i++) {
	    back[i] = args[i+step];
	}
	return back;
    }

    // It doesn't really parse the URLs it just creates a URL from a string
    // and then puts it into a URL array
    private static URL[] parseURLs(String spec) {
	Vector v = new Vector();
	try {
	    URL url = new URL(spec);
	    v.addElement(url);
	} catch (Exception ex) {
	    System.err.println("cannot create URL for "+spec);
	}
	URL back[] = new URL[v.size()];
	v.copyInto(back);
	return back;
    }

    private static void handleHSFile(File hsFile) {
	String path=null;
	hsName = hsFile.getName();
	if (hsName.endsWith(".jar")) {
	    // jar file
	    path = hsFile.getPath();
	    hsName = "";
	    try {
		JarFile jar = new JarFile(hsFile);
		Enumeration entries = jar.entries();
		while (entries.hasMoreElements()) {
		    ZipEntry entry = (ZipEntry) entries.nextElement();
		    String entryName = entry.getName();
		    if (entryName.endsWith(".hs")) {
			hsName = entryName;
			break;
		    }
		}
	    } catch (IOException ee) {
				// ingore it
	    }
	} else {
	    path = hsFile.getParent();
	    if (path == null) {
		path = File.separator;
	    } else {
		path = path.concat(File.separator);
	    }
	}
	if (path.startsWith("//")) {
	    path = path.substring(1);
	}

	if (path.startsWith("..")) {
	    String cpath = System.getProperty("user.dir");
	    path = cpath + File.separator + path;
	}

	hsPath = new String("file:" + path);
    }

    /**
     * We support two arguments; one of the two should be present (for now)
     * Later on, we will add a JHLauncher.data file, to be got using
     * getResource() that will have HelpSet, etc, etc.
     *
     * -helpset "helpset"	the HelpSet to look using the existing classloader
     * -classpath "classpath" 	A class path to use (default is the ClassLoader of JHLauncher)
     * -hsURL "URL to helpset" Where to look for the HelpSet.
     */
    public static void main(String args[]) {
	String path = null;	// the classpath to use (1.2-only)

	String hsSpec = null;

	// Initialize the loader
	loader = ClassLoader.getSystemClassLoader();

	// Get the arguments
	while (args.length > 0) {
	    if (args[0].equals("-helpset")) {
		// Name of HelpSet.  Look for it in the classpath
		args = shiftArgs(args, 1);
		File file = new File(args[0]);
		handleHSFile(file);
		URL urls[] = parseURLs(hsPath);
		// This is a 1.2 only feature.
		try {
		  loader = URLClassLoader.newInstance(urls, loader);
		} catch (NoClassDefFoundError err) {
		} catch (NoSuchMethodError err) {
		}
		args = shiftArgs(args, 1);
	    } else if (args[0].equals("-helppath")) {
		// This is a 1.2-only feature
		args = shiftArgs(args, 1);
		String hspath = args[0];
		if (hspath.startsWith("//")) {
		    hspath = hspath.substring(1);
		}

		if (hspath.startsWith("..")) {
		    String cpath = System.getProperty("user.dir");
		    hspath = cpath + File.separator + hspath;
		}

		hspath = new String("file:" + hspath);
		URL urls[] = parseURLs(hspath);
		args = shiftArgs(args, 1);
		try {
		    loader = URLClassLoader.newInstance(urls, loader);
		} catch (NoClassDefFoundError err) {
		} catch (NoSuchMethodError err) {
		}
	    } else if (args[0].equals("-hsURL")) {
		// This is the given URL for the HelpSet
		args = shiftArgs(args, 1);
		hsSpec = args[0];
		args = shiftArgs(args, 1);
	    } else if (args[0].equals("-id")) {
		// This is the given ID in a HelpSet to display
		args = shiftArgs(args, 1);
		id = args[0];
		args = shiftArgs(args, 1);
	    } else if (args[0].equals("-size")) {
		// This is the given ID in a HelpSet to display
		args = shiftArgs(args, 1);
		String size = args[0];
		int comma = size.indexOf(",");
		if (comma != -1) {
		    String width = size.substring(0,comma).trim();
		    String height = size.substring(comma+1).trim();
		    if (width != null && height != null) {
		        viewerWidth = Integer.parseInt(width);
			viewerHeight = Integer.parseInt(height);
		    }
		}
		args = shiftArgs(args, 1);
	    } else if (args[0].equals("-debug")) {
		debug = true;
		args = shiftArgs(args, 1);
	    } else {
		usage(null);
	    }
	}

	if (hsSpec != null) {
	    try {
		debug ("hsSpec=" + hsSpec);
		hsURL = new URL(hsSpec);
	    } catch (Exception ex) {
		usage("Invalid URL spec for HelpSet");
	    }
	}

	if (hsName != null && hsURL != null) {
	    usage(null);
	}

	if (hsName == null && hsURL == null) {
	    // This is stuff for later
	    URL resource;
	    if (loader == null) {
		resource = ClassLoader.getSystemResource("JHLauncher.data");
	    } else {
		resource = loader.getResource("JHLauncher.data");
	    }
	    if (resource != null) {
		System.err.println("Found JHLauncher.data at "+resource);
	    } else {
	        usage(null);
	    }
	}

	JHLauncher launcher = new JHLauncher();
    }

    protected static void usage(String msg) {
	if (msg != null) {
	    System.err.println("JHLauncher: "+msg);
	}
	System.err.println("Usage: [-helpset name | -helppath path | -hsURL spec | -id target | -size width,height ]");
	System.exit(1);
    }


    static boolean debug=false;
    static protected void debug(String msg) {
	if (debug) {
	    System.err.println("JHLauncher: "+msg);
	}
    }
}

